% Exemplary Solution for Exercise 1 - Assignment 1
% SNM HS2023, Jiwoo Oh, jiwoo.oh@chem.ethz.ch

% Clean Up
clear all; close all;

%% Assignment 1 : Conway's Game of Life

num_iter = 9; % number of time steps

N = 8; % grid size
board = zeros(N);

init = 2; % 1 for glider, 2 for beacon

% initial pattern: glider
if (init == 1) 
  board(1,1) = 1;
  board(2,2) = 1;
  board(2,3) = 1;
  board(3,1) = 1;
  board(3,2) = 1;
end

% initial pattern: beacon
if (init == 2)
  board(3,3) = 1;
  board(3,4) = 1;
  board(4,3) = 1;
  board(4,4) = 1;
  board(5,5) = 1;
  board(5,6) = 1;
  board(6,5) = 1;
  board(6,6) = 1;
end

% plotBoard.m should be in the same directory as this script mfile!
plotBoard (board);
for i = 1:num_iter
    board = nextGeneration(board);
    plotBoard(board)
end

%% Implementation
% Please refer to my comments on this exemplary code, make sure you also put
% comments on your own answer even though it looks obvious to you at the moment
% It will be not as obvious 3 months later, for example!

function nextboard = nextGeneration(board)
    % A function that receives cell array of nth generation and gives the array
    % of (n+1)th generation in return
    % Input : a cell array of any size
    % Output : a cell array of next generation that has evolved according
    % to the evolution rule
    [I,J] = size(board);
    countArray = countAliveNeighbor(board); % refer to comments on function 'countAliveNeighbor' in row 78
    nextboard = zeros(I,J); % output array (n+1)th generation

    % iteration over all elements of cell array
    for i = 1:I 
        for j = 1:J
            count = countArray(i,j); % number of alive neighbors of (i,j)-th element in Input cell array
            if board(i,j) == 0 && count == 3 % the dead cell
                % the dead cell becomes alive only when it has 3 alive neighbors
                nextboard(i,j) = 1;
            end
            if board(i,j) == 1 % the alive cell
                if count < 2 % the alive cell with fewer than 2 alive neighbors dies
                    nextboard(i,j) = 0;
                elseif (count == 2) || (count == 3) % the alive cell with 2 or 3 alive neighbors lives
                    nextboard(i,j) = 1;
                elseif count > 3 % the alive cell with more than 3 neighbors dies
                    nextboard(i,j) = 0;
                end
            end
        end
    end

end


function count = countAliveNeighbor(board)
    % A function that counts alive neighboars of a given cell array
    % Input : a the board array of any size
    % Output : an array, elements of which stands for number of alive
    % neighbors of the corresponding element of the input cell array
    % i.e. count(i,j) = number of neighboring 1's in board(i,j)
    % new_board = an array with one extended row/colum of zeros at each
    % boundary of the original array, so that there is no indexing error
    % for all elements of 'board'
    [I,J] = size(board);
    count = zeros(I,J);
    new_board = zeros(I+2,J+2);
    new_board(2:I+1,2:J+1) = board;
    for i = 1:I
        for j = 1:J
            for hor = -1:1
                for ver = -1:1
                    count(i,j) = count(i,j) + new_board(i+1+hor,j+1+ver);
                end
            end
            count(i,j) = count(i,j) - new_board(i+1,j+1); % cancel the addition for hor=0 && ver=0
        end
    end
end


function count = countAliveNeighbor2(board)
    % the same as above countAliveNeighbor function, another way to
    % implement using matrix convolution (func conv2)
    convarray = ones(3); % convolution array with ones except the center element, which corresponds to neighboring cells
    convarray(2,2) = 0; % convarray = [1 1 1; 1 0 1; 1 1 1]
    count = conv2(board,convarray,'same'); % conv2 function returns an array of the same size as 'board', where its element is the weighted sum of an element of 'board' and nearby pixels
end

